"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.XyoIterableStructure = void 0;
const xyo_buffer_1 = require("./xyo-buffer");
// eslint-disable-next-line import/no-cycle
const xyo_iterator_1 = require("./xyo-iterator");
const xyo_structure_1 = require("./xyo-structure");
class XyoIterableStructure extends xyo_structure_1.XyoStructure {
    static validate(structure) {
        const it = structure.newIterator();
        try {
            while (it.hasNext()) {
                const item = it.next().value;
                if (item instanceof XyoIterableStructure) {
                    if (!XyoIterableStructure.validate(item)) {
                        return false;
                    }
                }
            }
            return true;
        }
        catch (error) {
            return false;
        }
    }
    static toJson(structure) {
        const toReturn = [];
        if (structure instanceof XyoIterableStructure) {
            const it = structure.newIterator();
            while (it.hasNext()) {
                const item = it.next().value;
                const childArray = XyoIterableStructure.toJson(item);
                toReturn.push(childArray);
            }
        }
        else {
            toReturn.push(structure.getAll().getContentsCopy().toString('hex'));
        }
        return toReturn;
    }
    static newIterable(schema, items) {
        if (!schema.getIsIterable()) {
            throw new Error('Can not make iterable object from not iterable schema');
        }
        if (schema.getIsTypedIterable()) {
            const typedBuffer = XyoIterableStructure.encodeTyped(schema, items);
            return new XyoIterableStructure(xyo_structure_1.XyoStructure.encode(schema, new xyo_buffer_1.XyoBuffer(typedBuffer)));
        }
        const untypedBuffer = XyoIterableStructure.encodeUntyped(schema, items);
        return new XyoIterableStructure(xyo_structure_1.XyoStructure.encode(schema, new xyo_buffer_1.XyoBuffer(untypedBuffer)));
    }
    static encodeUntyped(schema, items) {
        const buffersToMerge = [];
        for (const item of items) {
            buffersToMerge.push(item.getAll().getContentsCopy());
        }
        return Buffer.concat(buffersToMerge);
    }
    static encodeTyped(schema, items) {
        const buffersToMerge = [];
        if (items.length < 1) {
            return Buffer.alloc(0);
        }
        const headerBuffer = Buffer.alloc(2);
        headerBuffer.writeUInt8(items[0].getSchema().encodingCatalogue, 0);
        headerBuffer.writeUInt8(items[0].getSchema().id, 1);
        buffersToMerge.push(headerBuffer);
        for (const item of items) {
            buffersToMerge.push(item.getAll().getContentsCopy().slice(2));
        }
        return Buffer.concat(buffersToMerge);
    }
    newIterator() {
        return new xyo_iterator_1.XyoIterator(this.readOwnHeader(), this, this.typedSchema !== undefined);
    }
    getCount() {
        const it = this.newIterator();
        let i = 0;
        while (it.hasNext()) {
            it.next();
            i++;
        }
        return i;
    }
    get(index) {
        const it = this.newIterator();
        let i = 0;
        while (it.hasNext()) {
            const item = it.next();
            if (index === i) {
                return item.value;
            }
            i++;
        }
        throw new Error(`Out of index ${index}`);
    }
    getId(id) {
        const itemsOfThatId = [];
        const it = this.newIterator();
        while (it.hasNext()) {
            const item = it.next().value;
            if (item.getSchema().id === id) {
                itemsOfThatId.push(item);
            }
        }
        return itemsOfThatId;
    }
    readItemAtOffset(offset) {
        if (this.typedSchema) {
            return this.readItemTyped(offset, this.typedSchema);
        }
        return this.readItemUntyped(offset);
    }
    addElement(element) {
        this.readOwnHeader();
        if (element.getSchema().getIsTypedIterable() && this.typedSchema) {
            if (element.getSchema().id === this.typedSchema.id) {
                const newBufferTyped = Buffer.concat([
                    this.getValue().getContentsCopy(),
                    element.getAll().getContentsCopy().slice(2),
                ]);
                this.contents = xyo_structure_1.XyoStructure.encode(element.getSchema(), new xyo_buffer_1.XyoBuffer(newBufferTyped));
                return;
            }
            throw new Error('Can not add different type to typed array');
        }
        const newBufferUntyped = Buffer.concat([
            this.getValue().getContentsCopy(),
            element.getAll().getContentsCopy(),
        ]);
        this.contents = xyo_structure_1.XyoStructure.encode(this.getSchema(), new xyo_buffer_1.XyoBuffer(newBufferUntyped));
    }
    readItemUntyped(offset) {
        const schema = this.readSchema(offset);
        const sizeOfObject = this.readSize(schema.getSizeIdentifier(), offset + 2);
        if (sizeOfObject === 0) {
            throw new Error('Size can not be 0');
        }
        const end = 2 + sizeOfObject + offset;
        this.checkIndex(end);
        const objectValue = xyo_buffer_1.XyoBuffer.wrap(this.contents, offset, end);
        if (schema.getIsIterable()) {
            return new XyoIterableStructure(objectValue);
        }
        return new xyo_structure_1.XyoStructure(objectValue);
    }
    readItemTyped(offset, schemaOfItem) {
        const sizeOfObject = this.readSize(schemaOfItem.getSizeIdentifier(), offset);
        if (sizeOfObject === 0) {
            throw new Error('Size can not be 0');
        }
        const end = sizeOfObject + offset;
        this.checkIndex(end);
        const objectValue = xyo_buffer_1.XyoBuffer.wrap(this.contents, offset, end);
        if (schemaOfItem.getIsIterable()) {
            return new XyoIterableStructure(objectValue, schemaOfItem);
        }
        return new xyo_structure_1.XyoStructure(objectValue, schemaOfItem);
    }
    readOwnHeader() {
        this.checkIndex(2);
        const schema = this.getSchema();
        this.checkIndex(2 + schema.getSizeIdentifier().valueOf() - 1);
        const totalSize = this.readSize(schema.getSizeIdentifier(), 2);
        if (!schema.getIsIterable()) {
            throw new Error('Object is not iterable');
        }
        if (schema.getIsTypedIterable() &&
            totalSize !== schema.getSizeIdentifier().valueOf()) {
            this.typedSchema = this.readSchema(schema.getSizeIdentifier().valueOf() + 2);
            return 4 + schema.getSizeIdentifier().valueOf();
        }
        return 2 + schema.getSizeIdentifier().valueOf();
    }
}
exports.XyoIterableStructure = XyoIterableStructure;
//# sourceMappingURL=xyo-iterable-structure.js.map