"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.XyoStructure = void 0;
const size_util_1 = require("./size-util");
const xyo_buffer_1 = require("./xyo-buffer");
const xyo_schema_1 = require("./xyo-schema");
const xyo_size_1 = require("./xyo-size");
class XyoStructure {
    constructor(contents, overrideSchema) {
        this.contents =
            contents instanceof Buffer
                ? (this.contents = new xyo_buffer_1.XyoBuffer(contents))
                : contents;
        this.overrideSchema = overrideSchema;
    }
    static encode(schema, value) {
        const bestSize = size_util_1.XyoSizeUtil.getBestSize(value.getSize());
        const header = Buffer.alloc(2 + bestSize.valueOf());
        const size = value.getSize() + bestSize.valueOf();
        const optimizedSchema = xyo_schema_1.XyoSchema.create(schema.id, schema.getIsIterable(), schema.getIsTypedIterable(), bestSize);
        header.writeUInt8(optimizedSchema.encodingCatalogue, 0);
        header.writeUInt8(schema.id, 1);
        switch (bestSize) {
            case xyo_size_1.XyoSize.ONE:
                header.writeUInt8(size, 2);
                break;
            case xyo_size_1.XyoSize.TWO:
                header.writeUInt16BE(size, 2);
                break;
            case xyo_size_1.XyoSize.FOUR:
                header.writeUInt32BE(size, 2);
                break;
            case xyo_size_1.XyoSize.EIGHT:
                header.writeUIntBE(size, 2, 8);
                break;
        }
        const encoded = Buffer.concat([header, value.getContentsCopy()]);
        return new xyo_buffer_1.XyoBuffer(encoded);
    }
    static newInstance(schema, value) {
        return new XyoStructure(XyoStructure.encode(schema, value));
    }
    getSchema() {
        if (this.overrideSchema) {
            return this.overrideSchema;
        }
        return this.readSchema(0);
    }
    getValue() {
        if (this.overrideSchema) {
            const schema = this.getSchema();
            const startOffset = schema.getSizeIdentifier().valueOf();
            const endOffset = this.readSize(schema.getSizeIdentifier(), 0);
            return this.contents.copyRangeOf(startOffset, endOffset);
        }
        const schema = this.getSchema();
        const startOffset = schema.getSizeIdentifier().valueOf() + 2;
        const endOffset = this.readSize(schema.getSizeIdentifier(), 2) + 2;
        return this.contents.copyRangeOf(startOffset, endOffset);
    }
    getAll() {
        if (this.overrideSchema) {
            const headerBuffer = Buffer.alloc(2);
            headerBuffer.writeUInt8(this.overrideSchema.encodingCatalogue, 0);
            headerBuffer.writeUInt8(this.overrideSchema.id, 1);
            const together = Buffer.concat([
                headerBuffer,
                this.contents.getContentsCopy(),
            ]);
            return new xyo_buffer_1.XyoBuffer(together);
        }
        return this.contents;
    }
    readSchema(offset) {
        this.checkIndex(offset + 2);
        const id = this.contents.getUInt8(1 + offset);
        const encodingCatalogue = this.contents.getUInt8(offset);
        return new xyo_schema_1.XyoSchema(id, encodingCatalogue);
    }
    checkIndex(index) {
        if (index > this.contents.getSize()) {
            throw new Error(`Out of count ${index}`);
        }
    }
    readSize(size, offset) {
        switch (size) {
            case xyo_size_1.XyoSize.ONE:
                return this.contents.getUInt8(offset);
            case xyo_size_1.XyoSize.TWO:
                return this.contents.getUInt16BE(offset);
            case xyo_size_1.XyoSize.FOUR:
                return this.contents.getUInt32BE(offset);
            case xyo_size_1.XyoSize.EIGHT:
                return this.contents.getUInt64BE(offset);
        }
    }
}
exports.XyoStructure = XyoStructure;
//# sourceMappingURL=xyo-structure.js.map