"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.XyoJsonBoundWitnessCreator = void 0;
const xyo_zig_zag_bound_witness_1 = require("../bound-witness/xyo-zig-zag-bound-witness");
const object_model_1 = require("../object-model");
const xyo_origin_state_1 = require("../origin/xyo-origin-state");
const schema_1 = require("../schema");
const xyo_ram_origin_state_repository_1 = require("./xyo-ram-origin-state-repository");
const xyo_stub_signature_1 = require("./xyo-stub-signature");
class XyoJsonBoundWitnessCreator {
    constructor() {
        this.partyToState = new Map();
    }
    // tslint:disable-next-line:prefer-array-literal
    createBlockfromJson(boundWitness) {
        const allKeys = Object.keys(boundWitness);
        if (allKeys.length > 2 || allKeys.length === 0) {
            throw new Error('Can only support two parties per BoundWitness');
        }
        if (allKeys.length === 2) {
            const alice = allKeys[0];
            const bob = allKeys[1];
            const aliceState = this.getPartyState(alice);
            const bobState = this.getPartyState(bob);
            const aliceHeuristics = this.createCustomHeuristicArrayWithDefaults([], alice);
            const bobHeuristics = this.createCustomHeuristicArrayWithDefaults([], bob);
            const boundWitnessAlice = new xyo_zig_zag_bound_witness_1.XyoZigZagBoundWitness(aliceState.getSigners(), aliceHeuristics, []);
            const boundWitnessBob = new xyo_zig_zag_bound_witness_1.XyoZigZagBoundWitness(bobState.getSigners(), bobHeuristics, []);
            const aliceToBobOne = boundWitnessAlice.incomingData(undefined, false);
            const bobToAliceOne = boundWitnessBob.incomingData(aliceToBobOne, true);
            const aliceToBobTwo = boundWitnessAlice.incomingData(bobToAliceOne, false);
            boundWitnessBob.incomingData(aliceToBobTwo, false);
            aliceState.addOriginBlock(boundWitnessAlice);
            bobState.addOriginBlock(boundWitnessBob);
            return boundWitnessAlice;
        }
        return this.createSelfSignedBoundWitness(allKeys[0]);
    }
    createBlocksFromJson(json) {
        const bwArray = [];
        const userData = JSON.parse(json);
        userData.forEach((boundWitness) => {
            const cBw = this.createBlockfromJson(boundWitness);
            if (cBw) {
                bwArray.push(cBw);
            }
        });
        this.partyToState = new Map();
        return bwArray;
    }
    createSelfSignedBoundWitness(party) {
        const charlieHeuristics = this.createCustomHeuristicArrayWithDefaults([], party);
        const charlieState = this.getPartyState(party);
        const boundWitnessCharlie = new xyo_zig_zag_bound_witness_1.XyoZigZagBoundWitness(charlieState.getSigners(), charlieHeuristics, []);
        boundWitnessCharlie.incomingData(undefined, true);
        return boundWitnessCharlie;
    }
    createCustomHeuristicArrayWithDefaults(heuristics, party) {
        const desiredHeuristics = Object.keys(heuristics);
        const returnedHeuristics = [];
        desiredHeuristics.forEach((heuristic) => {
            const newHeuristic = this.createHeuristic(heuristic, heuristics[heuristic]);
            if (newHeuristic) {
                returnedHeuristics.push(newHeuristic);
            }
        });
        return this.appendNeededHeuristics(returnedHeuristics, party);
    }
    createHeuristic(heuristic, value) {
        switch (heuristic) {
            case 'rssi': {
                const numBuff = Buffer.alloc(1);
                numBuff.writeInt8(value, 0);
                const buffer = new object_model_1.XyoBuffer(numBuff);
                return object_model_1.XyoStructure.newInstance(schema_1.XyoObjectSchema.RSSI, buffer);
            }
            case 'time': {
                const numBuff = Buffer.alloc(8);
                numBuff.writeUIntBE(value, 2, 6);
                const buffer = new object_model_1.XyoBuffer(numBuff);
                return object_model_1.XyoStructure.newInstance(schema_1.XyoObjectSchema.UNIX_TIME, buffer);
            }
        }
    }
    appendNeededHeuristics(heuristcs, party) {
        const state = this.getPartyState(party);
        if (!state) {
            return [];
        }
        const hash = state.getPreviousHash();
        if (hash) {
            heuristcs.push(hash);
        }
        heuristcs.push(state.getIndex());
        return heuristcs;
    }
    getPartyState(party) {
        let partyState = this.partyToState.get(party);
        if (!partyState) {
            const signer = new xyo_stub_signature_1.XyoStubSigner(Buffer.from(party));
            const stateRepo = new xyo_ram_origin_state_repository_1.XyoRamOriginStateRepository();
            const state = new xyo_origin_state_1.XyoOriginState(stateRepo);
            state.addSigner(signer);
            this.partyToState.set(party, state);
            partyState = this.partyToState.get(party);
        }
        return partyState;
    }
    setPartyState(party, state) {
        this.partyToState.set(party, state);
    }
}
exports.XyoJsonBoundWitnessCreator = XyoJsonBoundWitnessCreator;
//# sourceMappingURL=index.js.map